# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class AppendMarketDataType(str, Enum):
    """
    The supported complex market data types which can be appended to.
    """

    """
    allowed enum values
    """
    APPENDFXFORWARDCURVEBYQUOTEREFERENCE = 'AppendFxForwardCurveByQuoteReference'
    APPENDFXFORWARDCURVEDATA = 'AppendFxForwardCurveData'
    APPENDFXFORWARDPIPSCURVEDATA = 'AppendFxForwardPipsCurveData'
    APPENDFXFORWARDTENORCURVEDATA = 'AppendFxForwardTenorCurveData'
    APPENDFXFORWARDTENORPIPSCURVEDATA = 'AppendFxForwardTenorPipsCurveData'

    @classmethod
    def from_json(cls, json_str: str) -> AppendMarketDataType:
        """Create an instance of AppendMarketDataType from a JSON string"""
        return AppendMarketDataType(json.loads(json_str))
