# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


import json
import pprint
import re  # noqa: F401
from aenum import Enum, no_arg





class OperandType(str, Enum):
    """
    OperandType
    """

    """
    allowed enum values
    """
    ABSOLUTE = 'Absolute'
    PROPERTY = 'Property'

    @classmethod
    def from_json(cls, json_str: str) -> OperandType:
        """Create an instance of OperandType from a JSON string"""
        return OperandType(json.loads(json_str))
