# coding: utf-8

"""
    LUSID API

    FINBOURNE Technology  # noqa: E501

    Contact: info@finbourne.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json


from typing import Any, Dict, List, Optional
from pydantic.v1 import StrictStr, Field, BaseModel, Field, conlist, constr 
from lusid.models.link import Link

class SideDefinition(BaseModel):
    """
    SideDefinition
    """
    side:  StrictStr = Field(...,alias="side", description="A unique label identifying the side definition.") 
    security:  StrictStr = Field(...,alias="security", description="The field or property key defining the side's security, or instrument.") 
    currency:  StrictStr = Field(...,alias="currency", description="The field or property key defining the side's currency.") 
    rate:  StrictStr = Field(...,alias="rate", description="The field or property key defining the side's rate.") 
    units:  StrictStr = Field(...,alias="units", description="The value, field or property key defining the side's units.") 
    amount:  StrictStr = Field(...,alias="amount", description="The value, field or property key defining the side's amount") 
    notional_amount:  Optional[StrictStr] = Field(None,alias="notionalAmount", description="The value, field or property key defining the side's notional amount") 
    current_face:  Optional[StrictStr] = Field(None,alias="currentFace", description="The value, field or property key defining the side's current face / outstanding notional.") 
    links: Optional[conlist(Link)] = None
    __properties = ["side", "security", "currency", "rate", "units", "amount", "notionalAmount", "currentFace", "links"]

    class Config:
        """Pydantic configuration"""
        allow_population_by_field_name = True
        validate_assignment = True

    def __str__(self):
        """For `print` and `pprint`"""
        return pprint.pformat(self.dict(by_alias=False))

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.dict(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> SideDefinition:
        """Create an instance of SideDefinition from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self):
        """Returns the dictionary representation of the model using alias"""
        _dict = self.dict(by_alias=True,
                          exclude={
                          },
                          exclude_none=True)
        # override the default output from pydantic by calling `to_dict()` of each item in links (list)
        _items = []
        if self.links:
            for _item in self.links:
                if _item:
                    _items.append(_item.to_dict())
            _dict['links'] = _items
        # set to None if notional_amount (nullable) is None
        # and __fields_set__ contains the field
        if self.notional_amount is None and "notional_amount" in self.__fields_set__:
            _dict['notionalAmount'] = None

        # set to None if current_face (nullable) is None
        # and __fields_set__ contains the field
        if self.current_face is None and "current_face" in self.__fields_set__:
            _dict['currentFace'] = None

        # set to None if links (nullable) is None
        # and __fields_set__ contains the field
        if self.links is None and "links" in self.__fields_set__:
            _dict['links'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: dict) -> SideDefinition:
        """Create an instance of SideDefinition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return SideDefinition.parse_obj(obj)

        _obj = SideDefinition.parse_obj({
            "side": obj.get("side"),
            "security": obj.get("security"),
            "currency": obj.get("currency"),
            "rate": obj.get("rate"),
            "units": obj.get("units"),
            "amount": obj.get("amount"),
            "notional_amount": obj.get("notionalAmount"),
            "current_face": obj.get("currentFace"),
            "links": [Link.from_dict(_item) for _item in obj.get("links")] if obj.get("links") is not None else None
        })
        return _obj
